/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_DISPLAY_WINDOW_HPP
#define FGGL_DISPLAY_WINDOW_HPP

#include "fggl/modules/module.hpp"
#include "fggl/math/types.hpp"

#include "fggl/gfx/interfaces.hpp"

namespace fggl::display {

	class Window {
		public:
			virtual ~Window() = default;
			virtual void activate() const = 0;

			// window-related getters
			[[nodiscard]]
			virtual math::vec2i frameSize() const = 0;

			[[nodiscard]]
			virtual bool wantClose() const = 0;

			// window callbacks
			virtual void frameStart() = 0;
			virtual void frameEnd() = 0;

			gfx::Graphics &graphics() {
				return *m_graphics;
			}

			virtual void setTitle(const char *title) = 0;

			virtual void setFullscreen(bool state) = 0;

			[[nodiscard]]
			virtual bool isFullscreen() const = 0;

		protected:
			gfx::Graphics* m_graphics;
	};

	class WindowService {
		public:
			constexpr static const auto
				service = modules::make_service("fggl::display::WindowService");

			virtual Window *create() = 0;
			virtual void pollEvents() = 0;
	};

} // namespace fggl::display

#endif //FGGL_DISPLAY_WINDOW_HPP
